#!/usr/bin/env python
#
# laika prints a summary of the Launchpad bugs you touched this week
#
# man's best friend guides you through Launchpad
# 'Laika died within hours after launch' => an aptly named tool ;)
# http://en.wikipedia.org/wiki/Laika
#
# Copyright 2010 Alex Chiang <achiang@canonical.com>
# 
# This program is distributed under the terms of the
# GNU General Public License version 2.

import os
import re
import datetime

# Number of days prior to search for Launchpad activity
# TODO: make into a cmdline param
window = 8

now = datetime.datetime.utcnow()
my_bugs = {}

def print_header(header):
	print "==", header, "=="

# Here be side-effects. Using this interface adds the bug to global bug list.
def print_bugid(task):
	ago = ""
	my_bugs[task.bug.id] = 1
	delta = now - task.bug.date_last_updated.replace(tzinfo=None)
	if delta.days > 0:
		ago = "%d day%s" % (delta.days, "s" if delta.days > 1 else "")

	hours = delta.seconds / 3600
	if hours > 0:
		ago += ", " if ago else ""
		ago += "%d hour%s" % (hours, "s" if hours > 1 else "")
	
	minutes = (delta.seconds - (hours * 3600)) / 60
	if minutes > 0:
		ago += ", " if ago else ""
		ago += "%d minute%s" % (minutes, "s" if minutes > 1 else "")

	print task.title
	print "https://launchpad.net/bugs/" + str(task.bug.id)
	print "last updated", ago, "ago"

# heh.
# 	Timezones do not exist, all datetime objects have to be naive.
# 	aware means broken.
# http://www.enricozini.org/2009/debian/using-python-datetime/
def in_window(date):
	win = datetime.timedelta(window)
	date = date.replace(tzinfo=None)
	delta = now - date
	return delta <= win

def my_assignments(me):
	statuses = ['closed', 'fix_released', 'fix_committed',
		    'in_progress', 'triaged', 'confirmed', 'created']

	tasks = me.searchTasks(assignee=me)
	print_header("Assigned Bugs")

	for t in tasks:
		if my_bugs.has_key(t.bug.id):
			continue
		updates = []
		for s in statuses:
			attr = 'date_' + s
			date = getattr(t, attr)
			if not date:
				continue
			if in_window(date):
				updates.append("\t" + s + ": " + re.sub("\s.*$", "", str(date)))

		if updates:
			print_bugid(t)
			for u in updates:
				print u
			print
	print

def my_comments(me):
	tasks = me.searchTasks(bug_commenter=me)
	print_header("Commented Bugs")
	for t in tasks:
		if my_bugs.has_key(t.bug.id):
			continue
		for m in t.bug.messages:
			if m.owner_link != me.self_link:
				continue
			if in_window(m.date_created):
				print_bugid(t)
				print
				break
	print

def my_reported(me):
	tasks = me.searchTasks(bug_reporter=me)
	print_header("Reported Bugs")
	for t in tasks:
		if my_bugs.has_key(t.bug.id):
			continue
		if in_window(t.bug.date_created):
			print_bugid(t)
			print

# In order of decreasing importance, search and display activity on
# bugs assigned to me, then on bugs that I actually commented upon,
# and finally bugs that I just opened, but did nothing further with.
#
# Avoids duplication of activity. So if the status was changed on
# a bug assigned to you and you also commented on it, only report the
# status change.
def launch(lp):
	me = lp.me
	my_assignments(me)
	my_comments(me)
	my_reported(me)

if __name__ == "__main__":
	from launchpadlib.launchpad import Launchpad

	user = os.environ.get("USER")
	cachedir = "/home/" + user + "/.launchpadlib/cache/"
	lp = Launchpad.login_with('laika', 'production', cachedir)
	launch(lp)
